@@ -1,6 +1,5 @@
 from fastai.vision.all import *
 from fastai.metrics import accuracy
-import pandas as pd
 import numpy as np
 import os
 import PIL.Image as Image
@@ -28,19 +27,19 @@ def convertToImage(src, dst):
         img.save(dstPath)
     print('Files converted successfully')
     
-def loadData(valid_pct, get_items=get_image_files, get_y = parent_label, item_tfms = Resize(224, ResizeMethod.Pad, pad_mode='zeros')):
+def loadData(trainPath, valid_pct, bs=None, get_items=get_image_files, get_y = parent_label, item_tfms = Resize(224, ResizeMethod.Pad, pad_mode='zeros'), batch_tfms = aug_transforms()):
+    #aug_transforms(pad_mode='zeros', mult=2, min_scale=0.5)
     # parent_label -->> simply gets the name of the folder a file is in
     loader = DataBlock(
         blocks = (ImageBlock, CategoryBlock),
         get_items = get_items,
         splitter = RandomSplitter(valid_pct=valid_pct, seed=24),
         get_y = get_y,
-        #item_tfms = Resize(224, ResizeMethod.Pad, pad_mode='zeros'),   
-        #RandomResizedCrop(224), # by default it crops
         item_tfms = item_tfms,
-        batch_tfms=aug_transforms()
+        batch_tfms= batch_tfms
     )
-    return loader
+    dls = loader.dataloaders(trainPath, bs=bs)
+    return dls
 
 # dls = DataLoaders object, arch = architecture, path = path to where the trained model should be exported, epoch_ct = number of iterations, metrics = the metrics used to train the model, pretrained = whether or not to use a pretrained model (False = Create model from scratch)
 def trainModel(dls, arch, path, epoch_ct=1, metrics=error_rate, pretrained=True):
@@ -78,30 +77,31 @@ def confusionMatrix(model):
     interp.plot_confusion_matrix()
     plt.show()
 
-# model = the trained model, testPath = the path containing the test set of images, lbl_dict = the dictionary containing the labels, test_df = test set dataframe showing the actual class for each item
-def predict(model, testPath):
+# model = the trained model, testPath = the path containing the test set of images, labeled = whether or not the data has labels we can extract
+def predict(model, testPath, threshold=None, labeled=False):
     global modelaccuracy
     global modeltotal
     modelaccuracy = 0
     modeltotal = 0
+    warning = ''
     path = Path(testPath)
     dirs = os.listdir(path)
-    if(os.path.isdir(testPath+dirs[0])):
-        for dir in dirs: 
-            files = get_image_files(Path(testPath+dir))
-            for item in files:
-                # Predict each file
-                pred, pred_idx, probs = model.predict(item)
-                print(f"Item: {item} | Prediction: {pred}; Probability: {probs[pred_idx]:.04f}")
-                if(pred == parent_label(item)):
-                    modelaccuracy += 1
-                modeltotal += 1
-        try:
-            print("Total Accuracy", str((modelaccuracy/modeltotal) * 100)+"%")
-        except:
-            pass
-    else:
-        files = get_image_files(Path(testPath))
-        for item in files:
-            pred, pred_idx, probs = model.predict(item)
-            print(f"Item: {item} | Prediction: {pred}; Probability: {probs[pred_idx]:.04f}")
+    files = get_image_files(Path(testPath))
+    for item in files:
+        pred, pred_idx, probs = model.predict(item)
+        if(threshold is not None):
+            if(pred == 'goodware' and probs[pred_idx] < threshold):
+                pred = 'malware'
+                warning = '| this prediction was flipped'
+            elif(pred == 'malware' and probs[pred_idx] < threshold):
+                pred = 'goodware'
+                warning = '| this prediction was flipped'
+            else:
+                warning = ''
+        print(f"Item: {item} | Prediction: {pred}; Probability: {probs[pred_idx]:.04f} {warning}")
+        if(labeled):
+            if(pred == parent_label(item)):
+                modelaccuracy += 1
+            modeltotal += 1
+    if(labeled):
+        print("Total Accuracy", str((modelaccuracy/modeltotal) * 100)+"%")
